@@ -9,22 +9,24 @@ SPDX-License-Identifier: GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 #pragma once
 
 #include "abstractprojectitem.h"
+#include "project/transcodeseek.h"
 #include "utils/timecode.h"
 
 #include <KMessageWidget>
 
+#include <QActionGroup>
 #include <QDir>
 #include <QDomElement>
 #include <QFuture>
 #include <QLineEdit>
 #include <QListView>
+#include <QListWidget>
 #include <QMutex>
 #include <QPushButton>
+#include <QTimer>
 #include <QTreeView>
-#include <QListWidget>
 #include <QUrl>
 #include <QWidget>
-#include <QActionGroup>
 
 #include <KRecentDirs>
 
@@ -54,6 +56,7 @@ class QUndoCommand;
 class QVBoxLayout;
 class SmallJobLabel;
 class MediaBrowser;
+class MarkerListModel;
 
 namespace Mlt {
 class Producer;
@@ -74,11 +77,7 @@ protected:
     void mouseReleaseEvent(QMouseEvent *event) override;
     void mouseMoveEvent(QMouseEvent *event) override;
     void focusInEvent(QFocusEvent *event) override;
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
     void enterEvent(QEnterEvent *event) override;
-#else
-    void enterEvent(QEvent *event) override;
-#endif
     void leaveEvent(QEvent *event) override;
     void dropEvent(QDropEvent *event) override;
 
@@ -86,11 +85,13 @@ Q_SIGNALS:
     void focusView();
     void updateDragMode(PlaylistState::ClipState type);
     void displayBinFrame(QModelIndex ix, int frame, bool storeFrame = false);
-    void processDragEnd();
+    void performDrag(const QModelIndexList indexes);
+
 private:
     QPoint m_startPos;
     PlaylistState::ClipState m_dragType;
     QModelIndex m_lastHoveredItem;
+    QModelIndexList m_clickedIndexes;
 };
 
 /** @class MyTreeView
@@ -110,12 +111,9 @@ protected:
     void mouseReleaseEvent(QMouseEvent *event) override;
     void mouseMoveEvent(QMouseEvent *event) override;
     void focusInEvent(QFocusEvent *event) override;
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
     void enterEvent(QEnterEvent *) override;
-#else
-    void enterEvent(QEvent *event) override;
-#endif
     void leaveEvent(QEvent *event) override;
+    void dropEvent(QDropEvent *event) override;
 
 protected Q_SLOTS:
     void closeEditor(QWidget *editor, QAbstractItemDelegate::EndEditHint hint) override;
@@ -125,8 +123,8 @@ private:
     QPoint m_startPos;
     PlaylistState::ClipState m_dragType;
     QModelIndex m_lastHoveredItem;
+    QModelIndexList m_clickedIndexes;
     bool m_editing;
-    bool performDrag();
     bool isEditing() const;
 
 Q_SIGNALS:
@@ -136,6 +134,7 @@ Q_SIGNALS:
     void processDragEnd();
     void selectCurrent();
     void editingChanged();
+    void performDrag(const QModelIndexList indexes);
 };
 
 /** @class SmallJobLabel
@@ -211,7 +210,7 @@ public:
     void emitItemUpdated(std::shared_ptr<AbstractProjectItem> item);
 
     /** @brief Open a producer in the clip monitor */
-    void openProducer(std::shared_ptr<ProjectClip> controller);
+    void openProducer(std::shared_ptr<ProjectClip> controller, const QUuid &sequenceUuid = QUuid());
     void openProducer(std::shared_ptr<ProjectClip> controller, int in, int out);
 
     /** @brief Get a clip from it's id */
@@ -223,10 +222,10 @@ public:
     /** @brief Returns the frame size of a given clip. */
     QSize getFrameSize(int itemId) const;
     /** @brief Returns the state of a given clip: AudioOnly, VideoOnly, Disabled (Disabled means it has audio and video capabilities */
-    PlaylistState::ClipState getClipState(int itemId) const;
+    std::pair<PlaylistState::ClipState, ClipType::ProducerType> getClipState(int itemId) const;
 
-    /** @brief Add markers on clip \@param binId at \@param positions with @comments text if given */
-    void addClipMarker(const QString &binId, const QList<int> &positions, const QStringList &comments = {});
+    /** @brief Add markers on clip \@param binId at \@param markersData with @comments text if given */
+    void addClipMarker(const QString &binId, const QMap<int, QString> &markersData);
 
     /** @brief Get the count of all markers in all clips using this category */
     int getAllClipMarkers(int category) const;
@@ -268,11 +267,12 @@ public:
         @param ids a map of ids: {item id, {new parent, old parent}}
         @param redo if true, change to new parent, otherwise to old parent
     */
-    void doMoveClips(QMap<QString, std::pair<QString, QString>> ids, bool redo);
+    void doMoveClips(QMap<QString, std::pair<QString, QString>> ids, bool redo, bool dropFromSameSource);
     /** @brief Block/unblock signals from the bin selection model, useful for operations affecting many clips */
     void blockBin(bool block);
     void doMoveFolder(const QString &id, const QString &newParentId);
     void setupGeneratorMenu();
+    void clearMonitor();
 
     /** @brief Set focus to the Bin view. */
     void focusBinView();
@@ -313,7 +313,7 @@ public:
     /** @brief Return a list of all clips hashes used in this project */
     QStringList getProxyHashList();
     /** @brief Get binId of the current selected folder */
-    QString getCurrentFolder();
+    const QString getCurrentFolder();
     /** @brief Save a clip zone as MLT playlist */
     void saveZone(const QStringList &info, const QDir &dir);
     /** @brief A bin clip changed (its effects), invalidate preview */
@@ -331,12 +331,10 @@ public:
      * @param id the updated duration of the timeline clip
      * * @param current the uuid of the currently active timeline
      */
-    void updateSequenceClip(const QUuid &uuid, int duration, int pos, bool forceUpdate = false);
+    void updateSequenceClip(const QUuid &uuid, std::pair<int, int> durations, int pos, bool forceUpdate = false);
     /** @brief Update a sequence AV info (has audio/video) */
     void updateSequenceAVType(const QUuid &uuid, int tracksCount);
 
-    // TODO refac: remove this and call directly the function in ProjectItemModel
-    void cleanupUnused();
     void selectAll();
     /** @brief Save an mlt playlist from a bin id and a list of cuts
      * @param binId the id of the source clip for zones
@@ -384,6 +382,19 @@ public:
     const QString binInfoToString() const;
     /** @brief Load info (root folder, view type) from a string */
     const QString loadInfo(const QStringList binInfo, const QStringList existingNames);
+    /** @brief Display a clip effect stack */
+    void showItemEffectStack(ObjectId owner);
+    /** @brief Return the bin search widget */
+    QLineEdit *searchLine();
+    /** @brief Get marker models for all clips */
+    const QList<std::shared_ptr<MarkerListModel>> getAllClipsMarkers();
+    /** @brief Get the first selected clip*/
+    std::shared_ptr<ProjectClip> getFirstSelectedClip();
+    /** @brief Expand / collapse current item */
+    void expandCurrent();
+    /** @brief Expand / collapse all items */
+    void expandAll();
+    bool isMainBin() const;
 
 private Q_SLOTS:
     void slotAddClip();
@@ -453,6 +464,10 @@ private Q_SLOTS:
     void slotApplyFilters();
     /** @brief Open a new Bin widget */
     void slotOpenNewBin();
+    /** @brief Open clip in monitor */
+    void openClipInMonitor(std::shared_ptr<ProjectClip> clip, int in = -1, int out = -1, const QUuid &uuid = QUuid());
+    /** @brief Perform the drag */
+    bool performDrag(const QModelIndexList indexes);
 
 public Q_SLOTS:
 
@@ -466,9 +481,9 @@ public Q_SLOTS:
     void slotCreateProjectClip();
     void slotEditClipCommand(const QString &id, const QMap<QString, QString> &oldProps, const QMap<QString, QString> &newProps);
     /** @brief Start a filter job requested by a filter applied in timeline */
-    void slotStartFilterJob(const ItemInfo &info, const QString &id, QMap<QString, QString> &filterParams, QMap<QString, QString> &consumerParams,
+    void slotStartFilterJob(/*const ItemInfo &info,*/ const QString &id, QMap<QString, QString> &filterParams, QMap<QString, QString> &consumerParams,
                             QMap<QString, QString> &extraParams);
-    void slotItemDropped(const QStringList ids, const QModelIndex parent);
+    void slotItemDropped(const QStringList ids, const QModelIndex parent, bool dropFromSameSource);
     const QString slotUrlsDropped(const QList<QUrl> urls, const QModelIndex parent);
     void slotEffectDropped(const QStringList &effectData, const QModelIndex &parent);
     void slotTagDropped(const QString &tag, const QModelIndex &parent);
@@ -483,9 +498,10 @@ public Q_SLOTS:
     void slotUpdateClipProperties(const QString &id, const QMap<QString, QString> &properties, bool refreshPropertiesPanel);
     /** @brief Add effect to active Bin clip (used when double clicking an effect in list). */
     void slotAddEffect(std::vector<QString> ids, const QStringList &effectData);
-    void slotExpandUrl(const ItemInfo &info, const QString &url, QUndoCommand *command);
+    void slotExpandUrl(/*const ItemInfo &info,*/ const QString &url, QUndoCommand *command);
     /** @brief Abort all ongoing operations to prepare close. */
     void abortOperations();
+    void doDisplaySimpleMessage(const QString &text, KMessageWidget::MessageType type);
     void doDisplayMessage(const QString &text, KMessageWidget::MessageType type, const QList<QAction *> &actions = QList<QAction *>(), bool showCloseButton = false, BinMessage::BinCategory messageCategory = BinMessage::BinCategory::NoMessage);
     void doDisplayMessage(const QString &text, KMessageWidget::MessageType type, const QString logInfo);
     /** @brief Select a clip in the Bin from its id. */
@@ -507,7 +523,8 @@ public Q_SLOTS:
     /** @brief Check if a clip profile matches project, propose switch otherwise */
     void slotCheckProfile(const QString &binId);
     /** @brief A non seekable clip was added to project, propose transcoding */
-    void requestTranscoding(const QString &url, const QString &id, int type, bool checkProfile, const QString &suffix = QString(), const QString &message = QString());
+    void requestTranscoding(const QString &id, TranscodeSeek::TranscodeInfo info, bool checkProfile, const QString &suffix = QString(),
+                            const QString &message = QString());
     /** @brief Display the transcode to edit friendly format for currenly selected bin clips */
     void requestSelectionTranscoding(bool forceReplace = false);
     /** @brief Build the project bin audio/video icons according to color theme */
@@ -591,6 +608,7 @@ private:
     QToolButton *m_filterButton;
     /** @brief The info widget for failed jobs. */
     KMessageWidget *m_infoMessage;
+    QTimer m_messageTimer;
     BinMessage::BinCategory m_currentMessage;
     QStringList m_errorLog;
     /** @brief Dialog listing invalid clips on load. */
@@ -611,11 +629,10 @@ private:
     long m_processedAudio;
     /** @brief Indicates whether audio thumbnail creation is running. */
     QFuture<void> m_audioThumbsThread;
-    QAction *addAction(const QString &name, const QString &text, const QIcon &icon, const QString &category = {});
+    QAction *addBinAction(const QString &name, const QString &text, const QIcon &icon, const QString &category = {});
     void setupAddClipAction(QMenu *addClipMenu, ClipType::ProducerType type, const QString &name, const QString &text, const QIcon &icon);
     /** @brief Get the QModelIndex value for an item in the Bin. */
     QModelIndex getIndexForId(const QString &id, bool folderWanted) const;
-    std::shared_ptr<ProjectClip> getFirstSelectedClip();
     void showSlideshowWidget(const std::shared_ptr<ProjectClip> &clip);
     void processAudioThumbs();
     void updateSortingAction(int ix);
@@ -642,7 +659,7 @@ Q_SIGNALS:
     void requesteInvalidRemoval(const QString &, const QString &, const QString &);
     /** @brief Analysis data changed, refresh panel. */
     void updateAnalysisData(const QString &);
-    void openClip(std::shared_ptr<ProjectClip> c, int in = -1, int out = -1);
+    void openClip(std::shared_ptr<ProjectClip> c, int in = -1, int out = -1, const QUuid sequenceUuid = QUuid());
     /** @brief Fill context menu with occurrences of this clip in timeline. */
     void findInTimeline(const QString &, QList<int> ids = QList<int>());
     void clipNameChanged(int, const QString);
@@ -654,5 +671,5 @@ Q_SIGNALS:
     /** @brief Update a timeline tab name on clip rename. */
     void updateTabName(const QUuid &, const QString &);
     /** @brief Some timeline sequence producers have been updated, refresh their occurrences. */
-    void requestUpdateSequences(QMap<QUuid, int> seqs);
+    void requestUpdateSequences(QMap<QUuid, std::pair<int, int>> seqs);
 };
