@@ -175,13 +175,15 @@ void Core::initGUI(const QString &MltPath, const QUrl &Url, const QString &clips
     connect(this, &Core::showConfigDialog, m_mainWindow, &MainWindow::slotShowPreferencePage);
 
     Bin *bin = new Bin(m_projectItemModel, m_mainWindow);
-    connect(bin, &Bin::requestShowClipProperties, bin, &Bin::showClipProperties);
+    connect(bin, &Bin::requestShowClipProperties, bin, &Bin::showClipProperties, Qt::QueuedConnection);
     m_mainWindow->addBin(bin, QString(), false);
 
     connect(m_projectItemModel.get(), &ProjectItemModel::refreshPanel, m_mainWindow->activeBin(), &Bin::refreshPanel);
     connect(m_projectItemModel.get(), &ProjectItemModel::refreshClip, m_mainWindow->activeBin(), &Bin::refreshClip);
     connect(m_projectItemModel.get(), &ProjectItemModel::itemDropped, m_mainWindow->activeBin(), &Bin::slotItemDropped, Qt::QueuedConnection);
-    connect(m_projectItemModel.get(), &ProjectItemModel::urlsDropped, m_mainWindow->activeBin(), &Bin::slotUrlsDropped, Qt::QueuedConnection);
+    connect(m_projectItemModel.get(), &ProjectItemModel::urlsDropped, this, [this](const QList<QUrl> urls, const QModelIndex parent) {
+        QMetaObject::invokeMethod(m_mainWindow->activeBin(), "slotUrlsDropped", Qt::QueuedConnection, Q_ARG(QList<QUrl>, urls), Q_ARG(QModelIndex, parent));
+    });
 
     connect(m_projectItemModel.get(), &ProjectItemModel::effectDropped, m_mainWindow->activeBin(), &Bin::slotEffectDropped);
     connect(m_projectItemModel.get(), &ProjectItemModel::addTag, m_mainWindow->activeBin(), &Bin::slotTagDropped);
@@ -192,6 +194,7 @@ void Core::initGUI(const QString &MltPath, const QUrl &Url, const QString &clips
     m_mainWindow->init();
 
     // Secondary bins
+    m_guiConstructed = true;
     for (int i = 1; i < KdenliveSettings::binsCount(); i++) {
         bin = new Bin(m_projectItemModel, m_mainWindow, false);
         m_mainWindow->addBin(bin, QString(), false);
@@ -253,9 +256,7 @@ void Core::initGUI(const QString &MltPath, const QUrl &Url, const QString &clips
         m_mainWindow->restore(1, false);
     }
     m_mainWindow->show();
-    // bin->slotUpdatePalette();
     Q_EMIT m_mainWindow->GUISetupDone();
-    m_guiConstructed = true;
     if (!Url.isEmpty()) {
         Q_EMIT loadingMessageNewStage(i18n("Loading project…"));
     }
@@ -342,7 +343,6 @@ QString Core::openExternalApp(QString appPath, QStringList args)
     if (pCore->packageType() == LinuxPackageType::AppImage) {
         // Strip appimage custom LD_LIBRARY_PATH...
         QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
-        qDebug() << "::: GOT ENV: " << env.value("LD_LIBRARY_PATH") << ", PATH: " << env.value("PATH");
         QStringList libPath = env.value(QStringLiteral("LD_LIBRARY_PATH")).split(QLatin1Char(':'), Qt::SkipEmptyParts);
         QStringList updatedLDPath;
         for (auto &s : libPath) {
@@ -369,7 +369,7 @@ QString Core::openExternalApp(QString appPath, QStringList args)
         }
         process.setProcessEnvironment(env);
     }
-    qDebug() << "Starting external app" << appPath << "with arguments" << args;
+    qCInfo(KDENLIVE_LOG) << "Starting external app" << appPath << "with arguments" << args;
     if (!process.startDetached()) {
         return process.errorString();
     }
@@ -628,10 +628,10 @@ bool Core::setCurrentProfile(const QString profilePath)
         m_timecode.setFormat(currentProfile->fps());
         profileChanged();
         if (m_guiConstructed) {
-            Q_EMIT m_mainWindow->updateRenderWidgetProfile();
             m_monitorManager->resetProfiles();
             Q_EMIT m_monitorManager->updatePreviewScaling();
             if (m_mainWindow->hasTimeline() && m_mainWindow->getCurrentTimeline() && m_mainWindow->getCurrentTimeline()->model()) {
+                Q_EMIT m_mainWindow->updateRenderWidgetProfile();
                 // m_mainWindow->getCurrentTimeline()->model()->updateProfile(getProjectProfile());
                 m_mainWindow->getCurrentTimeline()->model()->updateFieldOrderFilter(currentProfile);
                 checkProfileValidity();
@@ -1019,13 +1019,15 @@ void Core::loadingClips(int count, bool allowInterrupt)
 void Core::displayBinMessagePrivate(const QString &text, int type, const QList<QAction *> &actions, bool showClose, BinMessage::BinCategory messageCategory)
 {
     if (m_mainWindow) {
-        m_mainWindow->getBin()->doDisplayMessage(text, KMessageWidget::MessageType(type), actions, showClose, messageCategory);
+        activeBin()->doDisplayMessage(text, KMessageWidget::MessageType(type), actions, showClose, messageCategory);
     }
 }
 
 void Core::displayBinLogMessagePrivate(const QString &text, int type, const QString logInfo)
 {
-    m_mainWindow->getBin()->doDisplayMessage(text, KMessageWidget::MessageType(type), logInfo);
+    if (m_mainWindow) {
+        activeBin()->doDisplayMessage(text, KMessageWidget::MessageType(type), logInfo);
+    }
 }
 
 void Core::clearAssetPanel(int itemId)
@@ -1140,27 +1142,37 @@ void Core::updateItemKeyframes(ObjectId id)
 
 void Core::updateItemModel(ObjectId id, const QString &service, const QString &updatedParam)
 {
-    if (m_guiConstructed && id.type == KdenliveObjectType::TimelineClip && !m_mainWindow->getCurrentTimeline()->loading &&
-        service.startsWith(QLatin1String("fade"))) {
-        auto tl = m_mainWindow->getTimeline(id.uuid);
-        if (tl) {
-            bool startFade = service.startsWith(QLatin1String("fadein")) || service.startsWith(QLatin1String("fade_from_"));
-            QVector<int> roles;
-            if (startFade) {
-                roles = {TimelineModel::FadeInRole, TimelineModel::FadeInMethodRole};
-                if (updatedParam == QLatin1String("alpha") || updatedParam == QLatin1String("level")) {
-                    roles << TimelineModel::FadeInMethodRole;
-                }
-            } else {
-                roles = {TimelineModel::FadeOutRole, TimelineModel::FadeOutMethodRole};
-                if (updatedParam == QLatin1String("alpha") || updatedParam == QLatin1String("level")) {
-                    roles << TimelineModel::FadeOutMethodRole;
-                }
-            }
-            qDebug() << "==== UPDATING FADE ROLES";
-            tl->controller()->updateClip(id.itemId, roles);
+    if (!m_guiConstructed) {
+        return;
+    }
+
+    if (id.type != KdenliveObjectType::TimelineClip || m_mainWindow->getCurrentTimeline()->loading) {
+        return;
+    }
+
+    if (!service.startsWith(QLatin1String("fade"))) {
+        return;
+    }
+
+    auto tl = m_mainWindow->getTimeline(id.uuid);
+    if (!tl) {
+        return;
+    }
+
+    bool startFade = service.startsWith(QLatin1String("fadein")) || service.startsWith(QLatin1String("fade_from_"));
+    QVector<int> roles;
+    if (startFade) {
+        roles = {TimelineModel::FadeInRole, TimelineModel::FadeInMethodRole};
+        if (updatedParam == QLatin1String("alpha") || updatedParam == QLatin1String("level")) {
+            roles << TimelineModel::FadeInMethodRole;
+        }
+    } else { // endFade
+        roles = {TimelineModel::FadeOutRole, TimelineModel::FadeOutMethodRole};
+        if (updatedParam == QLatin1String("alpha") || updatedParam == QLatin1String("level")) {
+            roles << TimelineModel::FadeOutMethodRole;
         }
     }
+    tl->controller()->updateClip(id.itemId, roles);
 }
 
 void Core::showClipKeyframes(ObjectId id, bool enable)
@@ -1429,15 +1441,18 @@ int Core::audioChannels()
     return 2;
 }
 
-void Core::addGuides(const QMap<QUuid, QList<int>> &guides)
+void Core::addGuides(const QMap<QUuid, QMap<int, QString>> &guides)
 {
-    QMapIterator<QUuid, QList<int>> i(guides);
+    QMapIterator<QUuid, QMap<int, QString>> i(guides);
     while (i.hasNext()) {
         i.next();
         QMap<GenTime, QString> markers;
-        for (int pos : i.value()) {
-            GenTime p(pos, pCore->getCurrentFps());
-            markers.insert(p, pCore->currentDoc()->timecode().getDisplayTimecode(p, false));
+        QMap<int, QString> values = i.value();
+        int ix = 1;
+        for (auto j = values.cbegin(), end = values.cend(); j != end; ++j) {
+            GenTime p(j.key(), pCore->getCurrentFps());
+            markers.insert(p, j.value().isEmpty() ? i18n("Marker %1", ix) : j.value());
+            ix++;
         }
         auto timeline = m_mainWindow->getTimeline(i.key());
         if (timeline == nullptr) {
@@ -1530,6 +1545,7 @@ void Core::cleanup()
         timeRemapWidget()->selectedClip(-1, QUuid());
     }
     if (m_mainWindow && m_mainWindow->getCurrentTimeline()) {
+        guidesList()->clear();
         disconnect(m_mainWindow->getCurrentTimeline()->controller(), &TimelineController::durationChanged, m_projectManager,
                    &ProjectManager::adjustProjectDuration);
         m_mainWindow->getCurrentTimeline()->controller()->clipActions.clear();
@@ -1796,3 +1812,29 @@ void Core::openDocumentationLink(const QUrl &link)
     }
     QDesktopServices::openUrl(link);
 }
+
+std::pair<QString, int> Core::getSelectedClipAndOffset()
+{
+    if (!m_guiConstructed || !m_mainWindow->getCurrentTimeline()->controller()) {
+        return {};
+    }
+    int cid = m_mainWindow->getCurrentTimeline()->controller()->getMainSelectedClip();
+    if (cid == -1) {
+        // No selected clip
+        return {};
+    }
+    // Get position and in point
+    ObjectId id(KdenliveObjectType::TimelineClip, cid, m_mainWindow->getCurrentTimeline()->getUuid());
+    int offset = getItemPosition(id) - getItemIn(id);
+    return {getTimelineClipBinId(id), offset};
+}
+
+int Core::currentTimelineOffset()
+{
+    return currentDoc()->getSequenceProperty(currentDoc()->activeUuid, QStringLiteral("kdenlive:sequenceproperties.timecodeOffset")).toInt();
+}
+
+void Core::updateHwDecoding()
+{
+    setenv("MLT_AVFORMAT_HWACCEL", KdenliveSettings::hwDecoding().toUtf8().constData(), 1);
+}
