@@ -14,6 +14,7 @@ SPDX-License-Identifier: GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 #include "dialogs/wizard.h"
 #include "kdenlive_debug.h"
 #include "kdenlivesettings.h"
+#include "lib/localeHandling.h"
 #include "mainwindow.h"
 #include "render/renderrequest.h"
 #include <config-kdenlive.h>
@@ -164,12 +165,52 @@ static void resetConfig()
     }
 }
 
+class Application : public QApplication
+{
+public:
+    QUrl url;
+    Application(int &argc, char **argv)
+        : QApplication(argc, argv)
+    {
+    }
+
+protected:
+    bool event(QEvent *event) override
+    {
+        if (event->type() == QEvent::FileOpen) {
+            QFileOpenEvent *openEvent = static_cast<QFileOpenEvent *>(event);
+            url = QUrl::fromLocalFile(openEvent->file());
+            return true;
+        } else
+            return QApplication::event(event);
+    }
+};
+
 int main(int argc, char *argv[])
 {
     int result = EXIT_SUCCESS;
 #ifdef USE_DRMINGW
     ExcHndlInit();
 #endif
+
+#ifdef Q_OS_MAC
+    // Launcher and Spotlight on macOS are not setting this environment
+    // variable needed by setlocale() as used by MLT.
+    if (QProcessEnvironment::systemEnvironment().value(MLT_LC_NAME).isEmpty()) {
+        qputenv(MLT_LC_NAME, QLocale().name().toUtf8());
+
+        QLocale localeByName(QLocale(QLocale().language(), QLocale().script(), QLocale().territory()));
+        if (QLocale().decimalPoint() != localeByName.decimalPoint()) {
+            // If region's numeric format does not match the language's, then we run
+            // into problems because we told MLT and libc to use a different numeric
+            // locale than actually in use by Qt because it is unable to give numeric
+            // locale as a set of ISO-639 codes.
+            QLocale::setDefault(localeByName);
+            qputenv("LANG", QLocale().name().toUtf8());
+        }
+    }
+#endif
+
     // Force QDomDocument to use a deterministic XML attribute order
     QHashSeed::setDeterministicGlobalSeed();
 
@@ -198,7 +239,7 @@ int main(int argc, char *argv[])
     // trigger initialisation of proper icon theme
     KIconTheme::initTheme();
 
-    QApplication app(argc, argv);
+    Application app(argc, argv);
 
     // Default to org.kde.desktop style unless the user forces another style
     if (qEnvironmentVariableIsEmpty("QT_QUICK_CONTROLS_STYLE")) {
@@ -253,7 +294,7 @@ int main(int argc, char *argv[])
     }
 
     KAboutData aboutData(QByteArray("kdenlive"), i18n("Kdenlive"), KDENLIVE_VERSION, i18n("An open source video editor."), KAboutLicense::GPL_V3,
-                         i18n("Copyright © 2007–2024 Kdenlive authors"), otherText, QStringLiteral("https://kdenlive.org"));
+                         i18n("Copyright © 2007–2025 Kdenlive authors"), otherText, QStringLiteral("https://kdenlive.org"));
     // main developers (alphabetical)
     aboutData.addAuthor(i18n("Jean-Baptiste Mardelle"), i18n("MLT and KDE SC 4 / KF5 port, main developer and maintainer"), QStringLiteral("jb@kdenlive.org"));
     // active developers with major involvement
@@ -324,15 +365,14 @@ int main(int argc, char *argv[])
     parser.process(app);
     aboutData.processCommandLine(&parser);
 
-    QUrl url;
     QUrl renderUrl;
     QString presetName;
     if (parser.positionalArguments().count() != 0) {
         const QString inputFilename = parser.positionalArguments().at(0);
         const QFileInfo fileInfo(inputFilename);
-        url = QUrl(inputFilename);
-        if (fileInfo.exists() || url.scheme().isEmpty()) { // easiest way to detect "invalid"/unintended URLs is no scheme
-            url = QUrl::fromLocalFile(fileInfo.absoluteFilePath());
+        app.url = QUrl(inputFilename);
+        if (fileInfo.exists() || app.url.scheme().isEmpty()) { // easiest way to detect "invalid"/unintended URLs is no scheme
+            app.url = QUrl::fromLocalFile(fileInfo.absoluteFilePath());
         }
         if (parser.positionalArguments().count() > 1) {
             // Output render
@@ -347,7 +387,7 @@ int main(int argc, char *argv[])
     qApp->processEvents(QEventLoop::AllEvents);
 
     if (parser.isSet(renderOption)) {
-        if (url.isEmpty()) {
+        if (app.url.isEmpty()) {
             qCritical() << "You need to give a valid file if you want to render from the command line.";
             return EXIT_FAILURE;
         }
@@ -364,7 +404,7 @@ int main(int argc, char *argv[])
         if (!Core::build(packageType, true)) {
             return EXIT_FAILURE;
         }
-        pCore->initHeadless(url);
+        pCore->initHeadless(app.url);
         app.processEvents();
 
         // ensure we have a proper kdenlive_render path, particular important for AppImage
@@ -459,41 +499,6 @@ int main(int argc, char *argv[])
     KCrash::initialize();
 #endif
 
-    qmlRegisterUncreatableMetaObject(PlaylistState::staticMetaObject, // static meta object
-                                     "com.enums",                     // import statement
-                                     1, 0,                            // major and minor version of the import
-                                     "ClipState",                     // name in QML
-                                     "Error: only enums");
-    qmlRegisterUncreatableMetaObject(FileStatus::staticMetaObject, // static meta object
-                                     "com.enums",                  // import statement
-                                     1, 0,                         // major and minor version of the import
-                                     "ClipStatus",                 // name in QML
-                                     "Error: only enums");
-    qmlRegisterUncreatableMetaObject(ClipType::staticMetaObject, // static meta object
-                                     "com.enums",                // import statement
-                                     1, 0,                       // major and minor version of the import
-                                     "ProducerType",             // name in QML
-                                     "Error: only enums");
-    qmlRegisterUncreatableMetaObject(AssetListType::staticMetaObject, // static meta object
-                                     "com.enums",                     // import statement
-                                     1, 0,                            // major and minor version of the import
-                                     "AssetType",                     // name in QML
-                                     "Error: only enums");
-    qmlRegisterUncreatableMetaObject(ToolType::staticMetaObject, // static meta object
-                                     "com.enums",                // import statement
-                                     1, 0,                       // major and minor version of the import
-                                     "ProjectTool",              // name in QML
-                                     "Error: only enums");
-    qmlRegisterUncreatableMetaObject(KeyframeType::staticMetaObject, // static meta object
-                                     "com.enums",                    // import statement
-                                     1, 0,                           // major and minor version of the import
-                                     "KeyframeType",                 // name in QML
-                                     "Error: only enums");
-    qmlRegisterUncreatableMetaObject(MaskModeType::staticMetaObject, // static meta object
-                                     "com.enums",                    // import statement
-                                     1, 0,                           // major and minor version of the import
-                                     "MaskModeType",                 // name in QML
-                                     "Error: only enums");
     if (parser.value(mltLogLevelOption) == QStringLiteral("verbose")) {
         mlt_log_set_level(MLT_LOG_VERBOSE);
     } else if (parser.value(mltLogLevelOption) == QStringLiteral("debug")) {
@@ -509,7 +514,7 @@ int main(int argc, char *argv[])
         QObject::connect(pCore.get(), &Core::loadingMessageIncrease, &splash, &Splash::increaseProgressMessage, Qt::DirectConnection);
         QObject::connect(pCore.get(), &Core::loadingMessageHide, &splash, &Splash::clearMessage, Qt::DirectConnection);
         QObject::connect(pCore.get(), &Core::closeSplash, &splash, [&]() { splash.finish(pCore->window()); });
-        pCore->initGUI(parser.value(mltPathOption), url, clipsToLoad);
+        pCore->initGUI(parser.value(mltPathOption), app.url, clipsToLoad);
         result = app.exec();
     }
     Core::clean();
