@@ -17,6 +17,7 @@ SPDX-License-Identifier: GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 #include "kdenlivesettings.h"
 #include "lib/audio/audioStreamInfo.h"
 #include "profiles/profilemodel.hpp"
+#include "xml/xml.hpp"
 
 #include "core.h"
 #include "kdenlive_debug.h"
@@ -27,7 +28,7 @@ SPDX-License-Identifier: GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 
 std::shared_ptr<Mlt::Producer> ClipController::mediaUnavailable;
 
-ClipController::ClipController(const QString &clipId, const std::shared_ptr<Mlt::Producer> &producer)
+ClipController::ClipController(const QString &clipId, const std::shared_ptr<Mlt::Producer> &producer, const QDomElement &description)
     : selectedEffectIndex(1)
     , m_audioThumbCreated(false)
     , m_producerLock(QReadWriteLock::Recursive)
@@ -53,10 +54,23 @@ ClipController::ClipController(const QString &clipId, const std::shared_ptr<Mlt:
     if (m_properties) {
         m_hasMultipleVideoStreams = m_properties->property_exists("kdenlive:multistreams");
         setProducerProperty(QStringLiteral("kdenlive:id"), m_controllerBinId);
+        if (m_masterProducer->parent().property_exists("kdenlive:control_uuid")) {
+            const QString uuid = m_masterProducer->parent().get("kdenlive:control_uuid");
+            m_controlUuid = QUuid(uuid);
+        } else {
+            m_controlUuid = QUuid::createUuid();
+            m_properties->set("kdenlive:control_uuid", m_controlUuid.toString().toUtf8().constData());
+        }
         getInfoForProducer();
         checkAudioVideo();
     } else {
         m_producerLock.lockForWrite();
+        if (Xml::hasXmlProperty(description, QStringLiteral("kdenlive:control_uuid"))) {
+            const QString uuid = Xml::getXmlProperty(description, QStringLiteral("kdenlive:control_uuid"));
+            m_controlUuid = QUuid(uuid);
+        } else {
+            m_controlUuid = QUuid::createUuid();
+        }
     }
 }
 
@@ -93,19 +107,6 @@ void ClipController::addMasterProducer(const std::shared_ptr<Mlt::Producer> &pro
     QMapIterator<QString, QVariant> i(m_tempProps);
     while (i.hasNext()) {
         i.next();
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-        switch (i.value().type()) {
-        case QVariant::Int:
-            setProducerProperty(i.key(), i.value().toInt());
-            break;
-        case QVariant::Double:
-            setProducerProperty(i.key(), i.value().toDouble());
-            break;
-        default:
-            setProducerProperty(i.key(), i.value().toString());
-            break;
-        }
-#else
         switch (i.value().typeId()) {
         case QMetaType::Int:
             setProducerProperty(i.key(), i.value().toInt());
@@ -117,7 +118,6 @@ void ClipController::addMasterProducer(const std::shared_ptr<Mlt::Producer> &pro
             setProducerProperty(i.key(), i.value().toString());
             break;
         }
-#endif
     }
     m_tempProps.clear();
     int id = m_controllerBinId.toInt();
@@ -127,6 +127,9 @@ void ClipController::addMasterProducer(const std::shared_ptr<Mlt::Producer> &pro
         qCDebug(KDENLIVE_LOG) << "// WARNING, USING INVALID PRODUCER";
     } else {
         setProducerProperty(QStringLiteral("kdenlive:id"), m_controllerBinId);
+        if (!m_properties->property_exists("kdenlive:control_uuid")) {
+            m_properties->set("kdenlive:control_uuid", m_controlUuid.toString().toUtf8().constData());
+        }
         getInfoForProducer();
         checkAudioVideo();
         if (!m_hasMultipleVideoStreams && m_service.startsWith(QLatin1String("avformat")) && (m_clipType == ClipType::AV || m_clipType == ClipType::Video)) {
@@ -139,23 +142,25 @@ void ClipController::addMasterProducer(const std::shared_ptr<Mlt::Producer> &pro
                 snprintf(property, sizeof(property), "meta.media.%d.stream.type", ix);
                 QString type = m_properties->get(property);
                 if (type == QLatin1String("video")) {
-                    QString key = QString("meta.media.%1.codec.name").arg(ix);
+                    QString key = QStringLiteral("meta.media.%1.codec.name").arg(ix);
                     QString codec_name = m_properties->get(key.toLatin1().constData());
                     if (codec_name == QLatin1String("png")) {
                         // This is a cover image, skip
                         qDebug() << "=== FOUND PNG COVER ART STREAM: " << ix;
+                        setProducerProperty(QStringLiteral("kdenlive:coverartstream"), ix);
                         continue;
                     }
                     if (codec_name == QLatin1String("mjpeg")) {
-                        key = QString("meta.media.%1.stream.frame_rate").arg(ix);
+                        key = QStringLiteral("meta.media.%1.stream.frame_rate").arg(ix);
                         QString fps = m_properties->get(key.toLatin1().constData());
                         if (fps.isEmpty()) {
-                            key = QString("meta.media.%1.codec.frame_rate").arg(ix);
+                            key = QStringLiteral("meta.media.%1.codec.frame_rate").arg(ix);
                             fps = m_properties->get(key.toLatin1().constData());
                         }
                         if (fps == QLatin1String("90000")) {
                             // This is a cover image, skip
                             qDebug() << "=== FOUND MJPEG COVER ART STREAM: " << ix;
+                            setProducerProperty(QStringLiteral("kdenlive:coverartstream"), ix);
                             continue;
                         }
                     }
@@ -167,7 +172,7 @@ void ClipController::addMasterProducer(const std::shared_ptr<Mlt::Producer> &pro
             if (videoStreams.count() > 1) {
                 setProducerProperty(QStringLiteral("kdenlive:multistreams"), 1);
                 m_hasMultipleVideoStreams = true;
-                QMetaObject::invokeMethod(pCore->bin(), "processMultiStream", Qt::QueuedConnection, Q_ARG(const QString &, m_controllerBinId),
+                QMetaObject::invokeMethod(pCore->bin(), "processMultiStream", Qt::QueuedConnection, Q_ARG(QString, m_controllerBinId),
                                           Q_ARG(QList<int>, videoStreams), Q_ARG(QList<int>, audioStreams));
             }
         }
@@ -182,6 +187,12 @@ const QByteArray ClipController::producerXml(Mlt::Producer producer, bool includ
     if (!producer.is_valid()) {
         return QByteArray();
     }
+    Mlt::Service s(producer.parent());
+    Mlt::Properties retainList(mlt_properties(s.get_data("xml_retain")));
+    if (retainList.is_valid()) {
+        Mlt::Playlist playlist(mlt_playlist(retainList.get_data("main_bin")));
+        producer.set("xml_retain main_bin", playlist.get_service(), 0);
+    }
     c.set("time_format", "frames");
     if (!includeMeta) {
         c.set("no_meta", 1);
@@ -320,6 +331,7 @@ void ClipController::getInfoForProducer()
     } else if (m_service == QLatin1String("blipflash")) {
         // Mostly used for testing
         m_clipType = ClipType::AV;
+        audioIndex = 0;
     } else if (m_service == QLatin1String("glaxnimate")) {
         // Mostly used for testing
         m_clipType = ClipType::Animation;
@@ -352,10 +364,11 @@ void ClipController::buildAudioInfo(int audioIndex)
     }
     m_audioInfo = std::make_unique<AudioStreamInfo>(m_masterProducer, audioIndex, m_clipType == ClipType::Playlist || m_clipType == ClipType::Timeline);
     // Load stream effects
-    for (int stream : m_audioInfo->streams().keys()) {
-        QString streamEffect = m_properties->get(QString("kdenlive:stream:%1").arg(stream).toUtf8().constData());
+    const QMap<int, QString> streams = m_audioInfo->streams();
+    for (auto stream = streams.cbegin(), end = streams.cend(); stream != end; ++stream) {
+        QString streamEffect = m_properties->get(QStringLiteral("kdenlive:stream:%1").arg(stream.key()).toUtf8().constData());
         if (!streamEffect.isEmpty()) {
-            m_streamEffects.insert(stream, streamEffect.split(QChar('#')));
+            m_streamEffects.insert(stream.key(), streamEffect.split(QChar('#')));
         }
     }
 }
@@ -413,17 +426,16 @@ bool ClipController::isValid()
 // static
 const char *ClipController::getPassPropertiesList(bool passLength)
 {
-    if (!passLength) {
-        return "kdenlive:proxy,kdenlive:originalurl,kdenlive:multistreams,rotate,force_aspect_num,force_aspect_den,force_aspect_ratio,force_fps,force_"
-               "progressive,force_tff,threads,"
-               "force_"
-               "colorspace,set.force_full_luma,file_hash,autorotate,disable_exif,xmldata,vstream,astream,set.test_image,set.test_audio,ttl";
+    std::string props = "kdenlive:control_uuid,kdenlive:proxy,kdenlive:originalurl,kdenlive:multistreams,rotate,force_aspect_ratio,"
+                        "force_fps,force_"
+                        "progressive,force_tff,threads,"
+                        "force_"
+                        "colorspace,set.force_full_luma,templatetext,file_hash,autorotate,disable_exif,xmldata,vstream,astream,set.test_image,set."
+                        "test_audio,ttl,kdenlive:tags,kdenlive:markers,kdenlive:masks,kdenlive:rating";
+    if (passLength) {
+        props.append(",length");
     }
-    return "kdenlive:proxy,kdenlive:originalurl,kdenlive:multistreams,rotate,force_aspect_num,force_aspect_den,force_aspect_ratio,force_fps,force_progressive,"
-           "force_tff,threads,"
-           "force_"
-           "colorspace,set.force_full_luma,templatetext,file_hash,autorotate,disable_exif,xmldata,length,vstream,astream,set.test_image,set.test_audio,"
-           "ttl";
+    return strdup(props.c_str());
 }
 
 QMap<QString, QString> ClipController::getPropertiesFromPrefix(const QString &prefix, bool withPrefix)
@@ -477,6 +489,9 @@ void ClipController::updateProducer(const std::shared_ptr<Mlt::Producer> &produc
         // Pass properties from previous producer
         m_properties->pass_list(passProperties, passList);
         setProducerProperty(QStringLiteral("kdenlive:id"), m_controllerBinId);
+        if (!m_properties->property_exists("kdenlive:control_uuid")) {
+            m_properties->set("kdenlive:control_uuid", m_controlUuid.toString().toUtf8().constData());
+        }
         if (m_clipType != ClipType::Timeline) {
             // Timeline clips effect stack point to the tractor and are handled in ProjectClip::reloadTimeline
             m_effectStack->resetService(m_masterProducer);
@@ -640,8 +655,16 @@ double ClipController::originalFps() const
     if (!m_properties) {
         return 0;
     }
-    QString propertyName = QStringLiteral("meta.media.%1.stream.frame_rate").arg(m_videoIndex);
-    return m_properties->get_double(propertyName.toUtf8().constData());
+    double fps = 0.;
+    int fps_den = m_properties->get_int("meta.media.frame_rate_den");
+    if (fps_den > 0) {
+        fps = m_properties->get_double("meta.media.frame_rate_num") / fps_den;
+    }
+    if (fps == 0. && m_videoIndex > -1) {
+        QString propertyName = QStringLiteral("meta.media.%1.stream.frame_rate").arg(m_videoIndex);
+        return m_properties->get_double(propertyName.toUtf8().constData());
+    }
+    return fps;
 }
 
 QString ClipController::videoCodecProperty(const QString &property) const
@@ -739,6 +762,9 @@ ClipType::ProducerType ClipController::clipType() const
 
 const QSize ClipController::getFrameSize() const
 {
+    if (m_clipType == ClipType::Color) {
+        return pCore->getCurrentFrameSize();
+    }
     QReadLocker lock(&m_producerLock);
     if (m_masterProducer == nullptr) {
         return QSize();
@@ -764,6 +790,10 @@ const QSize ClipController::getFrameSize() const
     if (height == 0) {
         height = m_properties->get_int("height");
     }
+    double sar = m_properties->get_double("meta.media.aspect_ratio");
+    if (sar > 0.) {
+        width *= sar;
+    }
     return QSize(width, height);
 }
 
@@ -809,7 +839,7 @@ void ClipController::checkAudioVideo()
             m_hasVideo = true;
             break;
         }
-        if (m_clipType == ClipType::Timeline) {
+        if (m_clipType == ClipType::Timeline || m_clipType == ClipType::Playlist) {
             if (m_audioInfo == nullptr) {
                 if (m_hasAudio) {
                     buildAudioInfo(-1);
@@ -872,7 +902,7 @@ void ClipController::checkAudioVideo()
             } else {
                 qDebug() << "* * * *ERROR INVALID FRAME On test";
             }
-            if (m_clipType == ClipType::Playlist) {
+            if (m_clipType == ClipType::Timeline || m_clipType == ClipType::Playlist) {
                 if (m_audioInfo == nullptr) {
                     if (m_hasAudio) {
                         buildAudioInfo(-1);
@@ -958,7 +988,7 @@ void ClipController::clearBackupProperties()
             passProps << propName;
         }
     }
-    for (const QString &p : qAsConst(passProps)) {
+    for (const QString &p : std::as_const(passProps)) {
         m_properties->clear(p.toUtf8().constData());
     }
     m_properties->clear("kdenlive:original.backup");
@@ -1137,10 +1167,23 @@ const QString ClipController::getOriginalUrl()
     return path;
 }
 
+bool ClipController::supportsProxy() const
+{
+    switch (clipType()) {
+        case ClipType::Video:
+        case ClipType::AV:
+        case ClipType::Image:
+        case ClipType::Playlist:
+        case ClipType::SlideShow:
+            return true;
+        default:
+            return false;
+    }
+}
+
 bool ClipController::hasProxy() const
 {
-    QString proxy = getProducerProperty(QStringLiteral("kdenlive:proxy"));
-    // qDebug()<<"::: PROXY: "<<proxy<<" = "<<getProducerProperty(QStringLiteral("resource"));
+    const QString proxy = getProducerProperty(QStringLiteral("kdenlive:proxy"));
     return proxy.size() > 2 && proxy == getProducerProperty(QStringLiteral("resource"));
 }
 
@@ -1153,11 +1196,11 @@ bool ClipController::isFullRange() const
 {
     bool full = !qstrcmp(m_masterProducer->get("meta.media.color_range"), "full");
     for (int i = 0; !full && i < m_masterProducer->get_int("meta.media.nb_streams"); i++) {
-        QString key = QString("meta.media.%1.stream.type").arg(i);
+        QString key = QStringLiteral("meta.media.%1.stream.type").arg(i);
         QString streamType(m_masterProducer->get(key.toLatin1().constData()));
         if (streamType == "video") {
             if (i == m_masterProducer->get_int("video_index")) {
-                key = QString("meta.media.%1.codec.pix_fmt").arg(i);
+                key = QStringLiteral("meta.media.%1.codec.pix_fmt").arg(i);
                 QString pix_fmt = QString::fromLatin1(m_masterProducer->get(key.toLatin1().constData()));
                 if (pix_fmt.startsWith("yuvj")) {
                     full = true;
@@ -1170,29 +1213,35 @@ bool ClipController::isFullRange() const
     return full;
 }
 
-bool ClipController::removeMarkerCategories(QList<int> toRemove, const QMap<int, int> remapCategories, Fun &undo, Fun &redo)
+bool ClipController::removeMarkerCategories(const QList<int> toRemove, const QMap<int, int> remapCategories, Fun &undo, Fun &redo)
 {
     bool found = false;
     if (m_markerModel->rowCount() == 0) {
         return false;
     }
     for (int i : toRemove) {
-        QList<CommentedTime> toDelete = m_markerModel->getAllMarkers(i);
+        const QList<CommentedTime> toDelete = m_markerModel->getAllMarkers(i);
         if (!found && toDelete.count() > 0) {
             found = true;
         }
         if (remapCategories.contains(i)) {
             // Move markers to another category
             int newType = remapCategories.value(i);
-            for (CommentedTime c : toDelete) {
+            for (const CommentedTime &c : toDelete) {
                 m_markerModel->addMarker(c.time(), c.comment(), newType, undo, redo);
             }
         } else {
             // Delete markers
-            for (CommentedTime c : toDelete) {
+            for (const CommentedTime &c : toDelete) {
                 m_markerModel->removeMarker(c.time(), undo, redo);
             }
         }
     }
     return found;
 }
+
+std::shared_ptr<Mlt::Producer> ClipController::sequenceProducer(const QUuid &)
+{
+    QReadLocker lock(&m_producerLock);
+    return m_masterProducer;
+}
